﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2013 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using System.Collections;
using System.Collections.Generic;
using System.Linq;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.DataPreprocessing;
using HeuristicLab.Problems.DataAnalysis;

namespace HeuristicLab.DataPreprocessing {
  [Item("PreprocessingData", "Represents data used for preprocessing.")]
  public class PreprocessingData : NamedItem, IPreprocessingData {

    private IDictionary<string, IList> variableValues;

    private IList<string> variableNames;

    private IDictionary<string, int> variableNameIndices;

    private double trainingToTestRatio;

    private PreprocessingData(PreprocessingData original, Cloner cloner)
      : base(original, cloner) {
      variableValues = new Dictionary<string, IList>(variableValues);
      variableNameIndices = new Dictionary<string, int>(variableNameIndices);
    }
    
    public PreprocessingData(IDataAnalysisProblemData problemData)
      : base() {
      Name = "-";

      variableNames = new List<string>(problemData.Dataset.VariableNames));
      // create dictionary from variable name to index
      variableNameIndices = new Dictionary<string, int>();
      var variableNamesList = problemData.Dataset.VariableNames.ToList();
      for (int i = 0; i < variableNamesList.Count; i++) {
        variableNameIndices.Add(variableNamesList[i], i);
      }
      
      // copy values
      variableValues = new Dictionary<string, IList>();
      foreach (var variableName in problemData.Dataset.VariableNames) {
        if (problemData.Dataset.IsType<double>(variableName)) {
          variableValues[variableName] = problemData.Dataset.GetDoubleValues(variableName).ToList();
        } else if (problemData.Dataset.IsType<string>(variableName)) {
          variableValues[variableName] = CreateColumn<string>(problemData.Dataset, variableNameIndices[variableName], x => x);
        } else if (problemData.Dataset.IsType<DateTime>(variableName)) {
          variableValues[variableName] = CreateColumn<DateTime>(problemData.Dataset, variableNameIndices[variableName], x => DateTime.Parse(x));
        } else {
          throw new ArgumentException("The datatype of column " + variableName + " must be of type List<double>, List<string> or List<DateTime>");
        }
      }

      trainingToTestRatio = (double)problemData.TrainingPartition.Size / problemData.TestPartition.Size;
      Columns = problemData.Dataset.Columns;
      Rows = problemData.Dataset.Rows;
    }

    private static IList CreateColumn<T>(Dataset ds, int column, Func<string, T> selector) {
      var list = new List<T>(ds.Rows);
      for (int row = 0; row < ds.Rows; row++) {
        list[row] = selector(ds.GetValue(row, column));
      }
      return list;
    }

    #region NamedItem abstract Member Implementations

    public override IDeepCloneable Clone(Cloner cloner) {
      return new PreprocessingData(this, cloner);
    }

    #endregion

    #region IPreprocessingData Members

    public T GetCell<T>(int columnIndex, int rowIndex) {
      return (T)variableValues[variableNames[columnIndex]][rowIndex];
    }

    public void SetCell<T>(int columnIndex, int rowIndex, T value) {
      throw new NotImplementedException();
    }

    public T GetCell<T>(string variableName, int row) {
      throw new NotImplementedException();
    }

    public void SetCell<T>(string variableName, int row, T value) {
      throw new NotImplementedException();
    }

    public IEnumerable<T> GetValues<T>(int columnIndex) {
      throw new NotImplementedException();
    }

    public IEnumerable<T> GetValues<T>(string variableName) {
      throw new NotImplementedException();
    }

    public void SetValues<T>(int columnIndex, IEnumerable<T> values) {
      throw new NotImplementedException();
    }

    public void SetValues<T>(string variableName, IEnumerable<T> values) {
      throw new NotImplementedException();
    }

    public void InsertRow(int rowIndex) {
      throw new NotImplementedException();
    }

    public void DeleteRow(int rowIndex) {
      throw new NotImplementedException();
    }

    public void InsertColumn(string variableName, int columnIndex) {
      throw new NotImplementedException();
    }

    public void DeleteColumn(int columnIndex) {
      throw new NotImplementedException();
    }

    public void DeleteColumn(string variableName) {
      throw new NotImplementedException();
    }

    IEnumerable<string> IPreprocessingData.VariableNames {
      get { throw new NotImplementedException(); }
    }

    public bool IsType<T>(int columnIndex) {
      throw new NotImplementedException();
    }

    public bool IsType<T>(string variableName) {
      throw new NotImplementedException();
    }

    public int Columns {
      get;
      private set;
    }

    public int Rows {
      get;
      private set;
    }

    public void ExportTo(IDataAnalysisProblemData problemData) {
      throw new NotImplementedException();
    }

    #endregion
  }
}
